@@ -202,7 +202,6 @@ int Incremental(char *devname, int verbose, int runstop,
 	}
 	close (dfd); dfd = -1;
 
-	memset(&info, 0, sizeof(info));
 	st->ss->getinfo_super(st, &info, NULL);
 
 	/* 3/ Check if there is a match in mdadm.conf */
@@ -396,7 +395,6 @@ int Incremental(char *devname, int verbose, int runstop,
 				goto out;
 			}
 			close(dfd2);
-			memset(&info2, 0, sizeof(info2));
 			st2->ss->getinfo_super(st2, &info2, NULL);
 			st2->ss->free_super(st2);
 			if (info.array.level != info2.array.level ||
@@ -707,7 +705,7 @@ static int count_active(struct supertype *st, struct mdinfo *sra,
 	int cnt = 0;
 	__u64 max_events = 0;
 	char *avail = NULL;
-	int *best;
+	int *best = NULL;
 	char *devmap = NULL;
 	int numdevs = 0;
 	int devnum;
@@ -778,9 +776,10 @@ static int count_active(struct supertype *st, struct mdinfo *sra,
 				best[info.disk.raid_disk] = devnum;
 				st->ss->getinfo_super(st, bestinfo, NULL);
 			} else { /* info.events much bigger */
-				memset(avail, 0, info.disk.raid_disk);
+				memset(avail, 0, raid_disks);
 				max_events = info.events;
 				avail[info.disk.raid_disk] = 2;
+				best[info.disk.raid_disk] = devnum;
 				st->ss->getinfo_super(st, bestinfo, NULL);
 			}
 		}
@@ -1435,7 +1434,6 @@ static int Incremental_container(struct supertype *st, char *devname,
 	int suuid[4];
 	int sfd;
 
-	memset(&info, 0, sizeof(info));
 	st->ss->getinfo_super(st, &info, NULL);
 
 	if ((runstop > 0 && info.container_enough >= 0) ||
